package com.ozs.framework.config;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor;

import javax.annotation.Resource;

import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.springframework.boot.autoconfigure.task.TaskExecutionProperties;
import org.springframework.boot.autoconfigure.task.TaskExecutionProperties.Pool;
import org.springframework.context.annotation.Bean;
//import com.ozs.common.utils.Threads;
//import org.apache.commons.lang3.concurrent.BasicThreadFactory;
//import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import com.ozs.common.utils.Threads;

/**
 * 线程池配置
 *
 * @author hx
 **/
@Configuration
public class ThreadPoolConfig
{
	
	@Resource
	private TaskExecutionProperties taskExecutionProperties;
	
//    // 核心线程池大小
    private int corePoolSize = 50;
//
//    // 最大可创建的线程数
//    private int maxPoolSize = 200;
//
//    // 队列最大长度
//    private int queueCapacity = 1000;
//
//    // 线程池维护线程所允许的空闲时间
//    private int keepAliveSeconds = 300;

    @Bean(name = "threadPoolTaskExecutor")
    public ThreadPoolTaskExecutor threadPoolTaskExecutor()
    {
    	Pool pool = taskExecutionProperties.getPool();
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setMaxPoolSize(pool.getMaxSize());
        executor.setCorePoolSize(pool.getCoreSize());
        executor.setQueueCapacity(pool.getQueueCapacity());
        executor.setKeepAliveSeconds((int)pool.getKeepAlive().getSeconds());
        // 线程池对拒绝任务(无线程可用)的处理策略
        //executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        return executor;
    }

    /**
     * 执行周期性或定时任务
     */
    @Bean(name = "scheduledExecutorService")
    protected ScheduledExecutorService scheduledExecutorService()
    {
        return new ScheduledThreadPoolExecutor(corePoolSize,
                new BasicThreadFactory.Builder().namingPattern("schedule-pool-%d").daemon(true).build(),
                new ThreadPoolExecutor.CallerRunsPolicy())
        {
            @Override
            protected void afterExecute(Runnable r, Throwable t)
            {
                super.afterExecute(r, t);
                Threads.printException(r, t);
            }
        };
    }
}
